@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2017 The ANGLE Project Authors. All rights reserved.
+// Copyright 2017 The ANGLE Project Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -27,13 +27,13 @@ MSG kBlitDepthOrStencilFormatMismatch = "Depth/stencil buffer format combination
 MSG kBlitDimensionsOutOfRange = "BlitFramebuffer dimensions out of 32-bit integer range.";
 MSG kBlitExtensionDepthStencilWholeBufferBlit = "Only whole-buffer depth and stencil blits are supported by this extension.";
 MSG kBlitExtensionFormatMismatch = "Attempting to blit and the read and draw buffer formats don't match.";
-MSG kBlitExtensionFromInvalidAttachmentType = "Blits are only supported from 2D texture = renderbuffer or default framebuffer attachments in this extension.";
+MSG kBlitExtensionFromInvalidAttachmentType = "Blits are only supported from 2D texture, renderbuffer or default framebuffer attachments in this extension.";
 MSG kBlitExtensionLinear = "Linear blit not supported in this extension.";
 MSG kBlitExtensionMultisampledDepthOrStencil = "Multisampled depth/stencil blit is not supported by this extension.";
 MSG kBlitExtensionMultisampledWholeBufferBlit = "Only whole-buffer blit is supported from a multisampled read buffer in this extension.";
 MSG kBlitExtensionNotAvailable = "Blit extension not available.";
 MSG kBlitExtensionScaleOrFlip = "Scaling and flipping in BlitFramebufferANGLE not supported by this implementation.";
-MSG kBlitExtensionToInvalidAttachmentType = "Blits are only supported to 2D texture = renderbuffer or default framebuffer attachments in this extension.";
+MSG kBlitExtensionToInvalidAttachmentType = "Blits are only supported to 2D texture, renderbuffer or default framebuffer attachments in this extension.";
 MSG kBlitFeedbackLoop = "Blit feedback loop: the read and draw framebuffers are the same.";
 MSG kBlitFramebufferMissing = "Read and draw framebuffers must both exist for a blit to succeed.";
 MSG kBlitFromMultiview = "Attempt to read from a multi-view framebuffer.";
@@ -48,8 +48,8 @@ MSG kBlitOnlyNearestForNonColor = "Only nearest filtering can be used when blitt
 MSG kBlitSameImageColor = "Read and write color attachments cannot be the same image.";
 MSG kBlitSameImageDepthOrStencil = "Read and write depth stencil attachments cannot be the same image.";
 MSG kBlitToMultiview = "Attempt to write to a multi-view framebuffer.";
-MSG kBlitTypeMismatchFixedOrFloat = "If the read buffer contains fixed-point or floating-point values = the draw buffer must as well.";
-MSG kBlitTypeMismatchFixedPoint = "If the read buffer contains fixed-point values = the draw buffer must as well.";
+MSG kBlitTypeMismatchFixedOrFloat = "If the read buffer contains fixed-point or floating-point values, the draw buffer must as well.";
+MSG kBlitTypeMismatchFixedPoint = "If the read buffer contains fixed-point values, the draw buffer must as well.";
 MSG kBlitTypeMismatchSignedInteger = "If the read buffer contains signed integer values the draw buffer must as well.";
 MSG kBlitTypeMismatchUnsignedInteger = "If the read buffer contains unsigned integer values the draw buffer must as well.";
 MSG kBufferAlreadyMapped = "Buffer is already mapped.";
@@ -96,6 +96,7 @@ MSG kElementArrayNoBufferOrPointer = "No element array buffer and no pointer.";
 MSG kEnumNotSupported = "Enum is not currently supported.";
 MSG kEnumRequiresGLES30 = "Enum requires GLES 3.0";
 MSG kEnumRequiresGLES31 = "Enum requires GLES 3.1";
+MSG kES1or32Required = "OpenGL ES 1.x or 3.2 Required";
 MSG kES31Required = "OpenGL ES 3.1 Required";
 MSG kES3Required = "OpenGL ES 3.0 Required.";
 MSG kExceedsComputeWorkGroupCountX = "num_groups_x cannot be greater than MAX_COMPUTE_WORK_GROUP_COUNT[0]";
@@ -117,9 +118,10 @@ MSG kExceedsMaxVertexAttribBindings = "Index must be within [0, MAX_VERTEX_ATTRI
 MSG kExceedsMaxVertexAttribStride = "Stride must be within [0, MAX_VERTEX_ATTRIB_STRIDE).";
 MSG kExceedsNumExtensions = "Index must be within [0, NUM_EXTENSIONS).";
 MSG kExceedsNumRequestableExtensions = "Index must be within [0, NUM_REQUESTABLE_EXTENSIONS_ANGLE).";
-MSG kExpectedProgramName = "Expected a program name = but found a shader name.";
-MSG kExpectedShaderName = "Expected a shader name = but found a program name.";
+MSG kExpectedProgramName = "Expected a program name, but found a shader name.";
+MSG kExpectedShaderName = "Expected a shader name, but found a program name.";
 MSG kExtensionNotEnabled = "Extension is not enabled.";
+MSG kExtensionNotDisablable = "Extension is not disablable.";
 MSG kExtensionNotRequestable = "Extension is not requestable.";
 MSG kExternalTextureNotSupported = "External texture extension not enabled";
 MSG kFeedbackLoop = "Feedback loop formed between Framebuffer and active Texture.";
@@ -135,6 +137,7 @@ MSG kFramebufferTextureLayerIncorrectTextureType = "Texture is not a three-dimen
 MSG kGenerateMipmapNotAllowed = "Texture format does not support mipmap generation.";
 MSG kGenerateMipmapZeroSize = "Cannot generate mipmaps for a zero-size texture in a WebGL context.";
 MSG kGeometryShaderExtensionNotEnabled = "GL_EXT_geometry_shader extension not enabled.";
+MSG kGetImageExtensionNotEnabled = "GL_ANGLE_get_image extension not enabled.";
 MSG kGLES1Only = "GLES1-only function.";
 MSG kImageSizeMustBeZero = "imageSize must be 0 if no texture data is provided.";
 MSG kImageSizeTooSmall = "imageSize is too small.";
@@ -173,7 +176,7 @@ MSG kInvalidBufferUsage = "Invalid buffer usage enum.";
 MSG kInvalidClearMask = "Invalid mask bits.";
 MSG kInvalidClientState = "Invalid client vertex array type.";
 MSG kInvalidClipPlane = "Invalid clip plane.";
-MSG kInvalidCombinedImageUnit = "Specified unit must be in [GL_TEXTURE0 = GL_TEXTURE0 + GL_MAX_COMBINED_IMAGE_UNITS)";
+MSG kInvalidCombinedImageUnit = "Specified unit must be in [GL_TEXTURE0, GL_TEXTURE0 + GL_MAX_COMBINED_IMAGE_UNITS)";
 MSG kInvalidComponents = "Invalid components.";
 MSG kInvalidCompressedFormat = "Not a valid compressed texture format.";
 MSG kInvalidCompressedImageSize = "Invalid compressed image size.";
@@ -211,7 +214,7 @@ MSG kInvalidFogDensity = "Invalid fog density (must be nonnegative).";
 MSG kInvalidFogMode = "Invalid fog mode.";
 MSG kInvalidFogParameter = "Invalid fog parameter.";
 MSG kInvalidFormat = "Invalid format.";
-MSG kInvalidFormatCombination = "Invalid combination of format = type and internalFormat.";
+MSG kInvalidFormatCombination = "Invalid combination of format, type and internalFormat.";
 MSG kInvalidFragmentInputBinding = "No such binding.";
 MSG kInvalidFramebufferAttachmentParameter = "Invalid parameter name for framebuffer attachment.";
 MSG kInvalidFramebufferLayer = "Framebuffer layer cannot be less than 0 or greater than GL_MAX_FRAMEBUFFER_LAYERS_EXT.";
@@ -220,6 +223,7 @@ MSG kInvalidFramebufferTarget = "Invalid framebuffer target.";
 MSG kInvalidFramebufferTextureLevel = "Mipmap level must be 0 when attaching a texture.";
 MSG kInvalidHandleType = "Invalid handle type.";
 MSG kInvalidImageAccess = "access is not one of the supported tokens.";
+MSG kInvalidImageLayout = "Invalid image layout.";
 MSG kInvalidImageFormat = "format is not one of supported image unit formats.";
 MSG kInvalidIndentifier = "Invalid identifier.";
 MSG kInvalidIndirectOffset = "indirect must be a multiple of the size of uint in basic machine units.";
@@ -236,7 +240,7 @@ MSG kInvalidMemoryBarrierBit = "Invalid memory barrier bit.";
 MSG kInvalidMipLevel = "Level of detail outside of range.";
 MSG kInvalidMipLevels = "Invalid level count.";
 MSG kInvalidMultisampledFramebufferOperation = "Invalid operation on multisampled framebuffer";
-MSG kInvalidMultitextureUnit = "Specified unit must be in [GL_TEXTURE0 = GL_TEXTURE0 + GL_MAX_TEXTURE_UNITS)";
+MSG kInvalidMultitextureUnit = "Specified unit must be in [GL_TEXTURE0, GL_TEXTURE0 + GL_MAX_TEXTURE_UNITS)";
 MSG kInvalidName = "Invalid name.";
 MSG kInvalidNameCharacters = "Name contains invalid characters.";
 MSG kInvalidPathCoefficientsArray = "No coefficients array given.";
@@ -270,7 +274,7 @@ MSG kInvalidProgramInterface = "Invalid program interface.";
 MSG kInvalidProgramName = "Program object expected.";
 MSG kInvalidProgramResourceIndex = "Invalid program resource index.";
 MSG kInvalidProgramResourceProperty = "Invalid program resource property.";
-MSG kInvalidProjectionMatrix = "Invalid projection matrix. Left/right = top/bottom = near/far intervals cannot be zero = and near/far cannot be less than zero.";
+MSG kInvalidProjectionMatrix = "Invalid projection matrix. Left/right, top/bottom, near/far intervals cannot be zero, and near/far cannot be less than zero.";
 MSG kInvalidPropCount = "Invalid propCount.";
 MSG kInvalidPropertyForProgramInterface = "Not an allowed program resource property for this program interface";
 MSG kInvalidProvokingVertex = "Invalid provoking vertex.";
@@ -323,17 +327,19 @@ MSG kInvalidTransformFeedbackName = "name is not a valid transform feedback.";
 MSG kInvalidType = "Invalid type.";
 MSG kInvalidUniformCount = "Only array uniforms may have count > 1.";
 MSG kInvalidUniformLocation = "Invalid uniform location";
-MSG kInvalidUnpackAlignment = "Unpack alignment must be 1 = 2 = 4 = or 8.";
+MSG kInvalidUnpackAlignment = "Unpack alignment must be 1, 2, 4 or 8.";
 MSG kInvalidVaryingLocation = "Location exceeds max varying.";
 MSG kInvalidVertexArray = "Vertex array does not exist.";
 MSG kInvalidVertexArrayName = "name is not a valid vertex array.";
 MSG kInvalidVertexAttribSize2101010 = "Type is INT_2_10_10_10_REV or UNSIGNED_INT_2_10_10_10_REV and size is not 4.";
-MSG kInvalidVertexAttrSize = "Vertex attribute size must be 1 = 2 = 3 = or 4.";
+MSG kInvalidVertexAttribSize1010102 = "Type is INT_10_10_10_2_OES or UNSIGNED_INT_10_10_10_2_OES and size is not 3 or 4.";
+MSG kInvalidVertexAttrSize = "Vertex attribute size must be 1, 2, 3, or 4.";
 MSG kInvalidVertexPointerSize = "Size for built-in vertex attribute is outside allowed range.";
 MSG kInvalidVertexPointerStride = "Invalid stride for built-in vertex attribute.";
 MSG kInvalidVertexPointerType = "Invalid type for built-in vertex attribute.";
 MSG kInvalidWidth = "Invalid width.";
 MSG kInvalidWrapModeTexture = "Invalid wrap mode for texture type.";
+MSG kInvalidZOffset = "zoffset is larger than MAX_3D_TEXTURE_SIZE-1";
 MSG kLengthZero = "Length must not be zero.";
 MSG kLevelNotZero = "Texture level must be zero.";
 MSG kLightParameterOutOfRange = "Light parameter out of range.";
@@ -367,7 +373,7 @@ MSG kNegativeBaseViewIndex = "Negative baseViewIndex.";
 MSG kNegativeBufferSize = "Negative buffer size.";
 MSG kNegativeBufSize = "Invalid bufSize.";
 MSG kNegativeCount = "Negative count.";
-MSG kNegativeHeightWidthDepth = "Cannot have negative height = width = or depth.";
+MSG kNegativeHeightWidthDepth = "Cannot have negative height, width, or depth.";
 MSG kNegativeLayer = "Negative layer.";
 MSG kNegativeLength = "Negative length.";
 MSG kNegativeLevel = "Level is negative.";
@@ -399,7 +405,7 @@ MSG kObjectNotGenerated = "Object cannot be used because it has not been generat
 MSG kOffsetAlignment = "offset must be a multiple of 4.";
 MSG kOffsetAndSizeAlignment = "Offset and size must be multiple of 4.";
 MSG kOffsetMustBeMultipleOfType = "Offset must be a multiple of the passed in datatype.";
-MSG kOffsetMustBeMultipleOfUint = "Offset must be a multiple of the size = in basic machine units = of uint";
+MSG kOffsetMustBeMultipleOfUint = "Offset must be a multiple of sizeof(uint) in basic machine units.";
 MSG kOffsetOverflow = "Offset overflows texture dimensions.";
 MSG kOtherQueryActive = "Other query is active.";
 MSG kOutsideOfBounds = "Parameter outside of bounds.";
@@ -421,7 +427,7 @@ MSG kReadBufferNone = "Read buffer is GL_NONE.";
 MSG kReadBufferNotAttached = "Read buffer has no attachment.";
 MSG kRectangleTextureCompressed = "Rectangle texture cannot have a compressed format.";
 MSG kRelativeOffsetTooLarge = "relativeOffset cannot be greater than MAX_VERTEX_ATTRIB_RELATIVE_OFFSET.";
-MSG kRenderableInternalFormat = "SizedInternalformat must be color-renderable = depth-renderable = or stencil-renderable.";
+MSG kRenderableInternalFormat = "SizedInternalformat must be color-renderable, depth-renderable, or stencil-renderable.";
 MSG kRenderbufferNotBound = "A renderbuffer must be bound.";
 MSG kResourceMaxRenderbufferSize = "Desired resource size is greater than max renderbuffer size.";
 MSG kResourceMaxTextureSize = "Desired resource size is greater than max texture size.";
@@ -457,7 +463,7 @@ MSG kTextureWidthOrHeightOutOfRange = "Width and height must be less than or equ
 MSG kTransfomFeedbackAlreadyActive = "Transform feedback is already active.";
 MSG kTransformFeedbackActiveDelete = "Attempt to delete an active transform feedback.";
 MSG kTransformFeedbackActiveDuringLink = "Cannot link program while program is associated with an active transform feedback object.";
-MSG kTransformFeedbackBufferDoubleBound = "A transform feedback buffer that would be written to is also bound to a non-transform-feedback target = which would cause undefined behavior.";
+MSG kTransformFeedbackBufferDoubleBound = "A transform feedback buffer that would be written to is also bound to a non-transform-feedback target, which would cause undefined behavior.";
 MSG kTransformFeedbackBufferMultipleOutputs = "Transform feedback has a buffer bound to multiple outputs.";
 MSG kTransformFeedbackBufferTooSmall = "Not enough space in bound transform feedback buffers.";
 MSG kTransformFeedbackDoesNotExist = "Transform feedback object that does not exist.";
@@ -487,7 +493,7 @@ MSG kVertexShaderTypeMismatch = "Vertex shader input type does not match the typ
 MSG kViewportNegativeSize = "Viewport size cannot be negative.";
 MSG kViewsExceedMaxArrayLayers = "baseViewIndex+numViews cannot be greater than GL_MAX_ARRAY_TEXTURE_LAYERS.";
 MSG kWebgl2NameLengthLimitExceeded = "Location lengths must not be greater than 1024 characters.";
-MSG kWebglBindAttribLocationReservedPrefix = "Attributes that begin with 'webgl_' = or '_webgl_' are not allowed.";
+MSG kWebglBindAttribLocationReservedPrefix = "Attributes that begin with 'webgl_', or '_webgl_' are not allowed.";
 MSG kWebglNameLengthLimitExceeded = "Location name lengths must not be greater than 256 characters.";
 MSG kZeroBoundToTarget = "Zero is bound to target.";
 // clang-format on
