@@ -30,6 +30,7 @@
 
 #include <wpe/WebKitAuthenticationRequest.h>
 #include <wpe/WebKitBackForwardList.h>
+#include <wpe/WebKitColor.h>
 #include <wpe/WebKitContextMenu.h>
 #include <wpe/WebKitDefines.h>
 #include <wpe/WebKitEditorState.h>
@@ -37,19 +38,22 @@
 #include <wpe/WebKitFindController.h>
 #include <wpe/WebKitFormSubmissionRequest.h>
 #include <wpe/WebKitHitTestResult.h>
+#include <wpe/WebKitInputMethodContext.h>
 #include <wpe/WebKitJavascriptResult.h>
 #include <wpe/WebKitNavigationAction.h>
 #include <wpe/WebKitNotification.h>
+#include <wpe/WebKitOptionMenu.h>
 #include <wpe/WebKitPermissionRequest.h>
 #include <wpe/WebKitPolicyDecision.h>
+#include <wpe/WebKitRectangle.h>
 #include <wpe/WebKitScriptDialog.h>
 #include <wpe/WebKitSettings.h>
 #include <wpe/WebKitURIRequest.h>
 #include <wpe/WebKitUserContentManager.h>
+#include <wpe/WebKitUserMessage.h>
 #include <wpe/WebKitWebContext.h>
 #include <wpe/WebKitWebResource.h>
 #include <wpe/WebKitWebViewBackend.h>
-#include <wpe/WebKitColor.h>
 #include <wpe/WebKitWebViewSessionState.h>
 #include <wpe/WebKitWindowProperties.h>
 
@@ -241,6 +245,11 @@ struct _WebKitWebViewClass {
                                                     WebKitNotification          *notification);
     void           (* web_process_terminated)      (WebKitWebView               *web_view,
                                                     WebKitWebProcessTerminationReason reason);
+    gboolean       (* user_message_received)       (WebKitWebView               *web_view,
+                                                    WebKitUserMessage           *message);
+    gboolean       (* show_option_menu)            (WebKitWebView               *web_view,
+                                                    WebKitOptionMenu            *menu,
+                                                    WebKitRectangle             *rectangle);
 
     void (*_webkit_reserved0) (void);
     void (*_webkit_reserved1) (void);
@@ -248,8 +257,6 @@ struct _WebKitWebViewClass {
     void (*_webkit_reserved3) (void);
     void (*_webkit_reserved4) (void);
     void (*_webkit_reserved5) (void);
-    void (*_webkit_reserved6) (void);
-    void (*_webkit_reserved7) (void);
 };
 
 WEBKIT_API GType
@@ -283,6 +290,9 @@ webkit_web_view_is_ephemeral                         (WebKitWebView
 WEBKIT_API gboolean
 webkit_web_view_is_controlled_by_automation          (WebKitWebView             *web_view);
 
+WEBKIT_API WebKitAutomationBrowsingContextPresentation
+webkit_web_view_get_automation_presentation_type     (WebKitWebView             *web_view);
+
 WEBKIT_API WebKitWebsiteDataManager *
 webkit_web_view_get_website_data_manager             (WebKitWebView             *web_view);
 
@@ -525,6 +535,25 @@ WEBKIT_API void
 webkit_web_view_get_background_color                 (WebKitWebView               *web_view,
                                                       WebKitColor                 *color);
 
+WEBKIT_API void
+webkit_web_view_send_message_to_page                 (WebKitWebView               *web_view,
+                                                      WebKitUserMessage           *message,
+                                                      GCancellable                *cancellable,
+                                                      GAsyncReadyCallback          callback,
+                                                      gpointer                     user_data);
+
+WEBKIT_API WebKitUserMessage *
+webkit_web_view_send_message_to_page_finish          (WebKitWebView               *web_view,
+                                                      GAsyncResult                *result,
+                                                      GError                     **error);
+
+WEBKIT_API void
+webkit_web_view_set_input_method_context             (WebKitWebView               *web_view,
+                                                      WebKitInputMethodContext    *context);
+
+WEBKIT_API WebKitInputMethodContext *
+webkit_web_view_get_input_method_context             (WebKitWebView               *web_view);
+
 G_END_DECLS
 
 #endif
