@@ -26,6 +26,8 @@
 #include "config.h"
 #include "WebPageProxy.h"
 
+#include "EditorState.h"
+#include "InputMethodState.h"
 #include "PageClientImpl.h"
 #include "WebsiteDataStore.h"
 #include <WebCore/NotImplemented.h>
@@ -77,9 +79,26 @@ void WebsiteDataStore::platformRemoveRecentSearches(WallTime)
     notImplemented();
 }
 
-void WebPageProxy::updateEditorState(const EditorState&)
+void WebPageProxy::updateEditorState(const EditorState& editorState)
 {
-    notImplemented();
+    m_editorState = editorState;
+    if (!editorState.shouldIgnoreSelectionChanges)
+        pageClient().selectionDidChange();
+}
+
+void WebPageProxy::sendMessageToWebViewWithReply(UserMessage&& message, CompletionHandler<void(UserMessage&&)>&& completionHandler)
+{
+    static_cast<PageClientImpl&>(pageClient()).sendMessageToWebView(WTFMove(message), WTFMove(completionHandler));
+}
+
+void WebPageProxy::sendMessageToWebView(UserMessage&& message)
+{
+    sendMessageToWebViewWithReply(WTFMove(message), [](UserMessage&&) { });
+}
+
+void WebPageProxy::setInputMethodState(Optional<InputMethodState>&& state)
+{
+    static_cast<PageClientImpl&>(pageClient()).setInputMethodState(WTFMove(state));
 }
 
 } // namespace WebKit
